#ifndef LCD_H
#define LCD_H

#include <inttypes.h>
#include "avr_utils.h"

#define LCD_DIMX			320						// anzahl der punkte x
#define LCD_DIMY			240						// anzahl der punkte y
#define LCD_FRAMERATE		45						// LCD-Framerate (25Hz -70Hz)
#define LCD_CHEIGHT			12						// Hoehe eines zeichens
#define LCD_WCHAR			8						// anzahl der punkte pro zeichen (horizontal)
#define LCD_NCHAR			(LCD_DIMX/LCD_WCHAR)	// horizontal anzahl der zeichen
#define	LCD_LINES			(LCD_DIMY/LCD_CHEIGHT)	// anzahl der char-zeilen
#define LCD_BUFS			(LCD_NCHAR*LCD_LINES)	// anzahl der Buffer-bytes
#define LCD_HFREQ			(LCD_FRAMERATE*LCD_DIMX)	// hfreq
#define LCD_IRQTIME			(F_CPU/8)/LCD_HFREQ		// timer-set

#define LCD_LOAD			B,0						// latch (= hsync) 
#define LCD_CP				B,2						// clock
#define LCD_PWM				B,1						// pwm um kontrast zu steuern
#define LCD_FRAME			D,5						// Frame-Signal
#define LCD_OFF				D,6						// display off (nur in einigen displays vorhanden)
#define LCD_M				D,7						// M-Signal (ac fuer lcd-kristall-umschaltung) wird ev. vom display selbst erzeugt
#define LCD_DATA			PORTC					// lcd-datenbits (bit 0-3)

#define LCD_RECT			1						// enable rect draw
#define LCD_RECT_TEXT		1						// enable rect with text (only if LCD_RECT is set)
#define LCD_FILL_RECT		1						// enable filled-rect draw (only if LCD_RECT is set)
#define LCD_MONSTER			1						// enable monster-font
#define LCD_INT				1						// enable lcd calibrate
//#define LCD_SIMPLE_FONT		1						// enable sinple font (char 0x00-0x7f) remember section-size in makefile


#ifdef LCD_SIMPLE_FONT

#define LCD_FONT_SRL		0x00					// single rechteck links oben
#define LCD_FONT_SRLU		0x01					// single rechteck links unten
#define LCD_FONT_SRR		0x02					// single rechteck rechts oben
#define LCD_FONT_SRRU		0x03					// single rechteck rechts unten
#define LCD_FONT_SRHL		0x04					// single rechteck horizontal-line
#define LCD_FONT_SRVL		0x05					// single rechteck vertical-line

#define LCD_FONT_DRL		0x06					// double rechteck links oben
#define LCD_FONT_DRLU		0x07					// double rechteck links unten
#define LCD_FONT_DRR		0x08					// double rechteck rechts oben
#define LCD_FONT_DRRU		0x09					// double rechteck rechts unten
#define LCD_FONT_DRHL		0x0a					// double rechteck horizontal-line
#define LCD_FONT_DRVL		0x0b					// double rechteck vertical-line


#define LCD_FONT_BLOCK		0x0c					// block
#define LCD_FONT_BLOCKT		0x0d					// block top
#define LCD_FONT_BLOCKB		0x0e					// block bottom

#define LCD_FONT_FILL		0x0f					// fill-char

#define LCD_FONT_CHECK		0x12					// Checkbox not Checked
#define LCD_FONT_CHECK1		0x13					// Checkbox Checked
#define LCD_FONT_RADIO		0x11					// Radiobutton not Selected
#define LCD_FONT_RADIO1		0x10					// Radiobutton selected

#else

#define LCD_FONT_SRL		0xda					// single rechteck links oben
#define LCD_FONT_SRLU		0xc0					// single rechteck links unten
#define LCD_FONT_SRR		0xbf					// single rechteck rechts oben
#define LCD_FONT_SRRU		0xd9					// single rechteck rechts unten
#define LCD_FONT_SRHL		0xc4					// single rechteck horizontal-line
#define LCD_FONT_SRVL		0xb3					// single rechteck vertical-line

#define LCD_FONT_DRL		0xc9					// double rechteck links oben
#define LCD_FONT_DRLU		0xc8					// double rechteck links unten
#define LCD_FONT_DRR		0xbb					// double rechteck rechts oben
#define LCD_FONT_DRRU		0xbc					// double rechteck rechts unten
#define LCD_FONT_DRHL		0xcd					// double rechteck horizontal-line
#define LCD_FONT_DRVL		0xba					// double rechteck vertical-line


#define LCD_FONT_BLOCK		0xdb					// block
#define LCD_FONT_BLOCKT		0xdf					// block top
#define LCD_FONT_BLOCKB		0xdc					// block bottom

#define LCD_FONT_FILL		0xb1					// fill-char

#define LCD_FONT_CHECK		0x09					// Checkbox not Checked
#define LCD_FONT_CHECK1		0x0a					// Checkbox Checked
#define LCD_FONT_RADIO		0x08					// Radiobutton not Selected
#define LCD_FONT_RADIO1		0x07					// Radiobutton selected

#define LCD_FONT_SHIFT		0x1E					// Char for Shift
#define LCD_FONT_DEL		0x11					// Char for DEL
#define LCD_FONT_LEFT		0x1B					// Char for LEFT
#define LCD_FONT_RIGHT		0x1A					// Char for RIGHT
#endif

typedef union
{
	unsigned char  asByte[2];
	unsigned char  *ptr;
} uptr;

extern VoidFnct vsync_function;

extern volatile unsigned int	char_pos;
extern volatile unsigned char	lcd_famec;

extern unsigned char char_buffer[];

extern const unsigned char font[];

void lcd_init(void);
void lcd_clrscr(void);
void lcd_setscr(void);
void lcd_set_kontrast(unsigned char a);
void lcd_home(void);
void lcd_gotoxy(uint8_t x, uint8_t y);
void lcd_putc(char c);
void lcd_puts(const char *s);
void lcd_puts_p(const char *progmem_s);

#ifdef LCD_INT 
void lcd_int(int16_t z); // Routine zum ausgeben von Zahlen
#endif

#ifdef LCD_RECT 
void lcd_rect(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1);
void lcd_rect_dl(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1);
#ifdef LCD_FILL_RECT 
void lcd_fill(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1);
void lcd_fill_rect(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1);
void lcd_fill_rect_dl(unsigned char x,unsigned char y,unsigned char x1,unsigned char y1);
#endif
#ifdef LCD_RECT_TEXT
void lcd_rect_text(unsigned char x,unsigned char y,char *txt);
void lcd_rect_text_dl(unsigned char x,unsigned char y,char *txt);
#endif
#endif


#ifdef LCD_MONSTER 
void lcd_mputc(unsigned char c);
void lcd_mputs(const char *s);
#endif

//#define lcd_puts_P(__s)         lcd_puts_p(PSTR(__s))

#endif //LCD_H
