#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <stdlib.h>
#include "avr_utils.h"
#include "key.h"
#include "lcd.h"
#include "menu.h"

#define SCRATCH 16

char is_shift=0;
Menu_page *lastmenu;
m_ptr *to_handle=NULL;
char menu_keylist[] PROGMEM =		"1234567890\\|qwertzuiop+asdfghjkl#<yxcvbnm,.-"; 
char menu_keylist_shift[] PROGMEM =	"!\"@$%&/()=?`QWERTZUIOP*ASDFGHJKL\'>YXCVBNM;:_"; 
char menu_blinkc;


void menu_itoa(char *buf,int val)
{
	unsigned char scratch[SCRATCH];
	char *ptr;

	*buf='\0';
	if (((int)val) < 0)
	{
		val = - val;
		*buf = '-'; buf++; *buf=0;
	}

	ptr = (char *)(scratch + SCRATCH);
	*--ptr = 0;

	do 
	{
		char ch = val % 10 + '0';
		if (ch > '9')
			ch += 'a' - '9' - 1;
		*--ptr = ch;
		val /= 10;
	} while (val);
	strcpy(buf,ptr);
}

void menu_init(void)
{
	lastmenu=&MainMenu;
	menu_draw();
}

void menu_do_post(void)
{
	unsigned char fu;
	if(to_handle!=NULL)
	{
		fu=pgm_read_byte(&to_handle->ptr_rect.func);
		switch(fu)
		{
			case MENU_SLIDE:
				eeprom_write_word((unsigned int *)pgm_read_word(&to_handle->ptr_slide.eep),*(unsigned int *)pgm_read_word(&to_handle->ptr_slide.tmp));
				break;
		}
		to_handle=NULL;
	}
}

char menu_draw_key(unsigned char x, unsigned char y, unsigned char c)
{
	if(c!=0)
	{
		lcd_rect(x,y,x+2,y+2);
		lcd_gotoxy(x+1,y+1);
		lcd_putc(c);
		return 1;
	}
	return 0;
}

void menu_draw_keypad(void)
{
	unsigned char x,c_count;
	c_count=0;
	char *keyl;
	
	if(is_shift)
	{
		keyl=menu_keylist_shift;
	}
	else
	{
		keyl=menu_keylist;
	}

	
	for(x=0;x<12;x++)
	{
		if(menu_draw_key(x*3+2, 5, pgm_read_byte(&keyl[c_count]))) c_count++;
	}
	for(x=0;x<11;x++)
	{
		if(menu_draw_key(x*3+3, 8, pgm_read_byte(&keyl[c_count]))) c_count++;
	}
	for(x=0;x<10;x++)
	{
		if(menu_draw_key(x*3+4, 11, pgm_read_byte(&keyl[c_count]))) c_count++;
	}
	for(x=0;x<11;x++)
	{
		if(menu_draw_key(x*3+3, 14, pgm_read_byte(&keyl[c_count]))) c_count++;
	}

	menu_draw_key(0,11,LCD_FONT_SHIFT);
	menu_draw_key(35,11,LCD_FONT_SHIFT);
	lcd_rect_text(4,17,"        ");
	menu_draw_key(18,17,LCD_FONT_DEL);
	menu_draw_key(22,17,LCD_FONT_LEFT);
	menu_draw_key(26,17,LCD_FONT_RIGHT);
	lcd_rect_text(30,17,"Return");
}

char menu_keypad_pos(unsigned char kpx, unsigned char kpy)
{
	char *keyl;
	unsigned char x,c_count,x1;
	c_count=0;

	if(is_shift)
	{
		keyl=menu_keylist_shift;
	}
	else
	{
		keyl=menu_keylist;
	}

	for(x=0;x<12;x++)
	{
		x1=x*3+2;
		if(kpx>=x1 && kpx<=x1+2 && kpy>= 5 && kpy <= 7)
		{
			return pgm_read_byte(&keyl[c_count]);
		}
		c_count++;
	}

	for(x=0;x<11;x++)
	{
		x1=x*3+3;
		if(kpx>=x1 && kpx<=x1+2 && kpy>= 8 && kpy <= 10)
		{
			return pgm_read_byte(&keyl[c_count]);
		}
		c_count++;
	}

	for(x=0;x<10;x++)
	{
		x1=x*3+4;
		if(kpx>=x1 && kpx<=x1+2 && kpy>= 11 && kpy <= 13)
		{
			return pgm_read_byte(&keyl[c_count]);
		}
		c_count++;
	}

	for(x=0;x<11;x++)
	{
		x1=x*3+3;
		if(kpx>=x1 && kpx<=x1+2 && kpy>= 14 && kpy <= 16)
		{
			return pgm_read_byte(&keyl[c_count]);
		}
		c_count++;
	}

	if(kpy>= 11 && kpy <= 13)
	{
		if((kpx>=0 && kpx<=2) || (kpx>=35 && kpx<=37))
		{
			if(is_shift)
				is_shift=0;
			else
				is_shift=1;
			menu_draw_keypad();
			return 0;
		}
	}

	if(kpy>= 17 && kpy <= 19)
	{
		if(kpx>=4 && kpx<=14)
		{
			return ' ';
		}

		if(kpx>=18 && kpx<=20)
		{
			return 0x02;
		}

		if(kpx>=22 && kpx<=24)
		{
			return 0x03;
		}

		if(kpx>=26 && kpx<=28)
		{
			return 0x04;
		}

		if(kpx>=30 && kpx<=39)
		{
			return '\n';
		}
	}
	return 0;
}

void menu_strcpy(char *dst, char *src)
{
	while(*src)
	{
		*dst++=*src++;
	}
	*dst=0;
}

unsigned char menu_strlen(unsigned char *src)
{
	unsigned char i=0;
	while(*src++)
	{
		i++;
	}
	return i;
}

void menu_keypad(char *tx_buf, unsigned char len)
{
	unsigned char x,i;
	unsigned char kpx,kpy;
	unsigned char menu_char_pos=0;

	for(i=menu_strlen((unsigned char *)tx_buf);i<len;i++)
	{
		tx_buf[i]=' ';
	}
	tx_buf[len]=0;
	
	lcd_clrscr();
	lcd_rect_text_dl(5,1,tx_buf);
	menu_draw_keypad();

	while( 1 )                                                          // Endlosschleife
	{

		if(lcd_famec%32==0)
		{
			menu_blinkc=tx_buf[menu_char_pos];
			lcd_gotoxy(7+menu_char_pos,2);
			lcd_putc(LCD_FONT_BLOCK);
		}
		else if(lcd_famec%32==12)
		{
			lcd_gotoxy(7+menu_char_pos,2);
			lcd_putc(menu_blinkc);
		}

		if(get_key_press())
		{
			get_key_pos(&kpx,&kpy);
			x=menu_keypad_pos(kpx,kpy);
			if(x=='\n')
			{
				break;
			}
			else if(x==0x02&& menu_char_pos>0)
			{
				menu_char_pos--;
				for(i=menu_char_pos;i<len-1;i++)
				{
					tx_buf[i]=tx_buf[i+1];
				}
				tx_buf[len-1]=' ';
				menu_blinkc=tx_buf[menu_char_pos];
				lcd_rect_text_dl(5,1,tx_buf);
			}
			else if(x==0x03 && menu_char_pos>0)
			{
				lcd_gotoxy(7+menu_char_pos,2);
				lcd_putc(menu_blinkc);
				menu_char_pos--;
				menu_blinkc=tx_buf[menu_char_pos];
			}
			else if(x==0x04 && menu_char_pos<len-1)
			{
				lcd_gotoxy(7+menu_char_pos,2);
				lcd_putc(menu_blinkc);
				menu_char_pos++;
				menu_blinkc=tx_buf[menu_char_pos];
			}
			else if(x>=' ')
			{
				for(i=len-1;i>=menu_char_pos+1;i--)
				{
					tx_buf[i]=tx_buf[i-1];
				}
				tx_buf[menu_char_pos]=x;
				if(menu_char_pos<len-1)
				{
					menu_char_pos++;
				}
				menu_blinkc=tx_buf[menu_char_pos];
				lcd_rect_text_dl(5,1,tx_buf);
			}
		}
	}
	lcd_clrscr();
}

void menu_keypad_test(void)
{
	char buf[40];
	menu_strcpy(buf,"testtext");
	menu_keypad(buf,12);
	menu_draw();
	lcd_rect_text_dl(5,14,buf);
}

void menu_do(void)
{
	m_ptr	*en;
	Menu_page *mpage;
	unsigned char ix=0, x, y, x1, y1,fu,kpx,kpy,do_rect,iy,eev;
	long int a;
	char *bu;
	char buf[40];
	FuncPtr select;	// routine to call when selected 

	get_key_pos(&kpx,&kpy);

	// loop until all menue-parts scanned
	while((en=(m_ptr *)pgm_read_word(&lastmenu->entries[ix]))!=NULL)
	{
		mpage=NULL;
		select=NULL;
		do_rect=0;
		// get all global data
		fu=pgm_read_byte(&en->ptr_rect.func);
		x=pgm_read_byte(&en->ptr_rect.x);
		y=pgm_read_byte(&en->ptr_rect.y);

		switch(fu)
		{
			case MENU_MBUTTON:
			case MENU_MBUTTON_DL:
				x1=x+2+strlen_P((char *)pgm_read_word(&en->ptr_mbutton.text));
				y1=y+2;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_mbutton.select);
				// get pointer to page
				mpage=(Menu_page *)pgm_read_word(&en->ptr_mbutton.page);
				do_rect=1;
				break;

			case MENU_BUTTON:
			case MENU_BUTTON_DL:
				x1=x+2+strlen_P((char *)pgm_read_word(&en->ptr_button.text));
				y1=y+2;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_button.select);
				do_rect=1;
				break;

			case MENU_RECT:
			case MENU_RECT_DL:
			case MENU_RECT_FILL:
			case MENU_RECT_FILL_DL:
			case MENU_RECT_BLOCK:
				x1=x+pgm_read_byte(&en->ptr_rect.w);
				y1=y+pgm_read_byte(&en->ptr_rect.h);
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_rect.select);
				do_rect=1;
				break;

			case MENU_TEXT:
				x1=x+strlen_P((char *)pgm_read_word(&en->ptr_text.text));
				y1=y;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_text.select);
				do_rect=1;
				break;

			case MENU_SLIDE:
				x1=x+pgm_read_byte(&en->ptr_slide.w);
				y1=y+1;

				// is the click for this element?
				if(kpx>=x && kpx<=x1 && kpy>=y && kpy<=y1)
				{
					// get pointer to function
					select=(FuncPtr)pgm_read_word(&en->ptr_slide.select);
					key_count=1; // autorepeat slow

					a=(pgm_read_word(&en->ptr_slide.v_max)-pgm_read_word(&en->ptr_slide.v_min));
					a*=(key_transform_x()-key_calc_x(x));
					a/=key_calc_x(pgm_read_byte(&en->ptr_slide.w));
					a=a+pgm_read_word(&en->ptr_slide.v_min);
					if((int)a<(int)pgm_read_word(&en->ptr_slide.v_min)) a=pgm_read_word(&en->ptr_slide.v_min);
					if((int)a>(int)pgm_read_word(&en->ptr_slide.v_max)) a=pgm_read_word(&en->ptr_slide.v_max);
//lcd_gotoxy(x,y+2);
//lcd_int(a);
					*(int *)pgm_read_word(&en->ptr_slide.tmp)=a;
					to_handle=en;
					menu_draw_element(en);
					do_rect=1;
				}
				break;

#ifdef LCD_MONSTER
			case MENU_MTEXT:
				x1=x+(strlen_P((char *)pgm_read_word(&en->ptr_text.text))*5);
				y1=y+7;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_text.select);
				do_rect=1;
				break;
#endif

			case MENU_VAL:
				x1=x+12;
				y1=y+2;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_val.select);
				do_rect=1;
				break;

			case MENU_STRING:
				eev=(unsigned char )pgm_read_byte(&en->ptr_string.len);
				x1=x+2+eev;
				y1=y+2;
				// get pointer to function
				select=(FuncPtr)pgm_read_word(&en->ptr_val.select);
				do_rect=1;
				break;

			case MENU_RADIO:
				iy=0;
				while((bu=(char *)pgm_read_word(&en->ptr_radio.text[iy]))!=NULL)
				{
					x1=x+1+strlen_P(bu);
					y1=y+iy;
					if(kpx>=x && kpx<=x1 && kpy==y1)
					{
						select=(FuncPtr)pgm_read_word(&en->ptr_radio.select);
						y=y1;
						do_rect=1;
						eeprom_write_byte((unsigned char *)pgm_read_word(&en->ptr_radio.eep),iy);
						menu_draw_element(en);
						break;
					}
					iy++;
				}
				break;

			case MENU_CHECK:
				iy=0;
				while((bu=(char *)pgm_read_word(&en->ptr_radio.text[iy]))!=NULL)
				{
					x1=x+1+strlen_P(bu);
					y1=y+iy;
					if(kpx>=x && kpx<=x1 && kpy==y1)
					{
						eev=eeprom_read_byte((unsigned char *)pgm_read_word(&en->ptr_radio.eep));
						select=(FuncPtr)pgm_read_word(&en->ptr_radio.select);
						y=y1;
						do_rect=1;
						eev ^= (1<<iy);
						eeprom_write_byte((unsigned char *)pgm_read_word(&en->ptr_radio.eep),eev);
						menu_draw_element(en);
						break;
					}
					iy++;
				}
				break;
		}


		if(do_rect)
		{
			// is the click for this element?
			if(kpx>=x && kpx<=x1 && kpy>=y && kpy<=y1)
			{
				switch(fu)
				{
					case MENU_VAL:
						menu_itoa(buf,(int)eeprom_read_word((unsigned int *)pgm_read_word(&en->ptr_val.val)));
						for(iy=menu_strlen((unsigned char *)buf);iy<10;iy++)
						{
							buf[iy]=' ';
						}
						buf[10]=0;
						menu_keypad(buf,10);
						eeprom_write_word((unsigned int *)pgm_read_word(&en->ptr_val.val),atoi(buf));
						menu_draw();
						break;

					case MENU_STRING:
						y1=(unsigned char )pgm_read_byte(&en->ptr_string.len);
						eeprom_read_block(buf, (char *)pgm_read_word(&en->ptr_string.val), y1);  
						for(iy=menu_strlen((unsigned char *)buf);iy<y1;iy++)
						{
							buf[iy]=' ';
						}
						buf[y1]=0;
						menu_keypad(buf,10);
						eeprom_write_block(buf,(unsigned char *)pgm_read_word(&en->ptr_string.val),y1);
						menu_draw();
						break;
				}

				if(mpage!=NULL)
				{
					// if page is set... do it
					lastmenu=mpage;
					lcd_clrscr();
					menu_draw();
				}

				if(select!=NULL)
				{
					// if function is set... call it
					select();
				}

				// yes... stop prozessing
				break;
			}
		}
		do_rect=0;
		ix++;
	}
}

void menu_draw_element(m_ptr *en)
{
	long int v_long;
	unsigned char x, y, x1=0, y1=0,fu,eev,i;
	char buf[32];
	char *bu;

	fu=pgm_read_byte(&en->ptr_rect.func);
	x=pgm_read_byte(&en->ptr_rect.x);
	y=pgm_read_byte(&en->ptr_rect.y);

	switch(fu)
	{
		case MENU_MBUTTON_DL:
		case MENU_MBUTTON:
			strncpy_P(buf, (char *)pgm_read_word(&en->ptr_mbutton.text), sizeof(buf)-1);
			break;

		case MENU_BUTTON_DL:
		case MENU_BUTTON:
			strncpy_P(buf, (char *)pgm_read_word(&en->ptr_button.text), sizeof(buf)-1);
			break;

		case MENU_RECT:
		case MENU_RECT_DL:
		case MENU_RECT_FILL:
		case MENU_RECT_FILL_DL:
		case MENU_RECT_BLOCK:
			x1=x+pgm_read_byte(&en->ptr_rect.w);
			y1=y+pgm_read_byte(&en->ptr_rect.h);
			break;

		case MENU_SLIDE:
			x1=x+pgm_read_byte(&en->ptr_slide.w);
			y1=y+1;
			break;

#ifdef LCD_MONSTER
		case MENU_MTEXT:
#endif
		case MENU_TEXT:
			strncpy_P(buf, (char *)pgm_read_word(&en->ptr_text.text), sizeof(buf)-1);
			lcd_gotoxy(x,y);
			break;

		case MENU_VAL:
			menu_itoa(buf,(int)eeprom_read_word((unsigned int *)pgm_read_word(&en->ptr_val.val)));
			for(i=menu_strlen((unsigned char *)buf);i<10;i++)
			{
				buf[i]=' ';
			}
			buf[10]=0;
			break;

		case MENU_STRING:
		
			y1=(unsigned char )pgm_read_byte(&en->ptr_string.len);
			eeprom_read_block(buf, (char *)pgm_read_word(&en->ptr_string.val), y1);  
			for(i=menu_strlen((unsigned char *)buf);i<y1;i++)
			{
				buf[i]=' ';
			}
			buf[y1]=0;
			break;
	}

	switch(fu)
	{
		case MENU_BUTTON_DL:
		case MENU_MBUTTON_DL:
			lcd_rect_text_dl(x,y,buf);
			break;
		case MENU_BUTTON:
		case MENU_MBUTTON:
			lcd_rect_text(x,y,buf);
			break;
		case MENU_RECT:
			lcd_rect(x,y,x1,y1);
			break;
		case MENU_RECT_DL:
			lcd_rect_dl(x,y,x1,y1);
			break;
		case MENU_RECT_FILL:
			lcd_fill_rect(x,y,x1,y1);
			break;
		case MENU_RECT_FILL_DL:
			lcd_fill_rect_dl(x,y,x1,y1);
			break;
		case MENU_RECT_BLOCK:
			lcd_fill(x,y,x1,y1);
			break;
		case MENU_TEXT:
			lcd_puts(buf);
			break;

#ifdef LCD_MONSTER
		case MENU_MTEXT:
			lcd_mputs(buf);
			break;
#endif

		case MENU_STRING:
		case MENU_VAL:
			lcd_rect_text(x,y,buf);
			break;

		case MENU_SLIDE:
			lcd_rect(x,y,x1,y1);
			v_long=(int)pgm_read_byte(&en->ptr_slide.w);
			v_long*=(int)(*(int *)pgm_read_word(&en->ptr_slide.tmp)-(int)pgm_read_word(&en->ptr_slide.v_min));
			eev=v_long/((int)pgm_read_word(&en->ptr_slide.v_max)-(int)pgm_read_word(&en->ptr_slide.v_min));

			if(eev<0) eev=0;
			if(eev>pgm_read_byte(&en->ptr_slide.w)) eev=pgm_read_byte(&en->ptr_slide.w);

			lcd_gotoxy(x+eev,y);
			lcd_putc(LCD_FONT_BLOCK);
			lcd_gotoxy(x+eev,y1);
			lcd_putc(LCD_FONT_BLOCK);
			break;

		case MENU_RADIO:
			x1=0;
			y1=0;
			eev=eeprom_read_byte((unsigned char *)pgm_read_word(&en->ptr_radio.eep));

			while((bu=(char *)pgm_read_word(&en->ptr_radio.text[x1]))!=NULL)
			{
				if(strlen_P(bu)>y1) y1=strlen_P(bu);
				lcd_gotoxy(x,y+x1);
				if(x1==eev)
				{
					lcd_putc(LCD_FONT_RADIO1);
				}
				else
				{
					lcd_putc(LCD_FONT_RADIO);
				}
				lcd_gotoxy(x+2,y+x1);
				strncpy_P(buf,bu, sizeof(buf)-1);
				lcd_puts(buf);
				x1++;
			}
			break;

		case MENU_CHECK:
			x1=0;
			y1=0;
			eev=eeprom_read_byte((unsigned char *)pgm_read_word(&en->ptr_radio.eep));

			while((bu=(char *)pgm_read_word(&en->ptr_radio.text[x1]))!=NULL)
			{
				if(strlen_P(bu)>y1) y1=strlen_P(bu);
				lcd_gotoxy(x,y+x1);
				if(eev&1)
				{
					lcd_putc(LCD_FONT_CHECK1);
				}
				else
				{
					lcd_putc(LCD_FONT_CHECK);
				}
				lcd_gotoxy(x+2,y+x1);
				strncpy_P(buf,bu, sizeof(buf)-1);
				lcd_puts(buf);
				eev=eev>>1;
				x1++;
			}
			break;
	}
}

void menu_draw(void)
{
	m_ptr	*en;
	unsigned char ix=0;
	unsigned char fu;

	// draw all elements of this menue-page
	while((en=(m_ptr *)pgm_read_word(&lastmenu->entries[ix]))!=NULL)
	{
		fu=pgm_read_byte(&en->ptr_rect.func);

		switch(fu)
		{
			case MENU_SLIDE:
				*(int *)pgm_read_word(&en->ptr_slide.tmp)=(int)eeprom_read_word((unsigned int *)pgm_read_word(&en->ptr_slide.eep));
				break;
		}

		menu_draw_element(en);
		ix++;
	}
}


